require 'execjs'
require 'time'
require 'net/http'
require 'json'

module Api
  module V2
    class SystemInfoController < ApplicationController
      def index
        system_info = {
          timestamp: Time.now,
          ruby_version: RUBY_VERSION,
          rails_version: Rails.version,
          os: RbConfig::CONFIG['host_os'],
          cpu_architecture: RbConfig::CONFIG['host_cpu'],
          hostname: `hostname`.strip,
          uptime: `uptime`.strip,
          memory_usage: `free -m`.strip,
          disk_usage: `df -h`.strip,
          network_interfaces: network_interfaces,
          internet_connectivity: internet_connectivity?,
          cpu_usage: cpu_usage,
          load_average: load_average,
          processes: processes,
          open_ports: open_ports,
          query: fetch_druid_data
        }

        render json: system_info
      end

      private

      def network_interfaces
        `ip addr`.strip
      end

      def internet_connectivity?
        system('ping -c 1 google.com > /dev/null 2>&1')
      end

      def cpu_usage
        `top -bn1 | grep "Cpu(s)" | sed "s/.*, *\([0-9.]*\)%* id.*/\\1/" | awk '{print 100 - $1"%"}'`.strip
      end

      def load_average
        `uptime | awk -F'load average:' '{ print $2 }' | xargs`.strip
      end

      def processes
        `ps aux --sort=-%mem | head -n 10`.strip
      end

      def open_ports
        `ss -tuln`.strip
      end

      def fetch_druid_data
        data_sources = fetch_data_sources_from_sensor

        results = data_sources.map do |data_source, _name|
          query = build_query(data_source)
          send_query_to_druid(query)
        end

        # render json: { results: results }
        results
      end

      def fetch_data_sources_from_sensor
        sensors = Sensor.where(domain_type: Sensor::NAMESPACE).pluck(:uuid, :name)
        sensors.map { |uuid, name| ["rb_flow_#{uuid}", name] }.unshift(["rb_flow", "root"])
      end

      def build_query(data_source)
        start_time = (Time.now - 7 * 24 * 60 * 60).iso8601
        end_time = Time.now.iso8601
        {
          queryType: "topN",
          dataSource: data_source,
          granularity: "day",
          dimensions: "sensor",
          threshold: 25,
          context: {
            timeout: 90000,
            skipEmptyBuckets: true
          },
          intervals: ["#{start_time}/#{end_time}"],
          aggregations: [
            { "type": "longSum", "name": "bytes", "fieldName": "sum_bytes" }
          ]
        }
      end

      def druid_url
        @druid_url ||= retrieve_druid_broker_url
      end
    
      # Fetches the Druid broker URL from Zookeeper
      def retrieve_druid_broker_url
        zk_data = YAML.load(File.read('config/rbdruid_config.yml'))[Rails.env].symbolize_keys
        zk_connect = zk_data[:zk_connect]
        discovery_path = '/druid/discoveryPath/broker'
    
        brokers = fetch_brokers(zk_connect, discovery_path)
        raise "No active Druid Broker found." if brokers.empty?
    
        brokers.first
      end
    
      # Queries Zookeeper for active brokers
      def fetch_brokers(zk_connect, discovery_path)
        brokers_names = []
    
        ZK.open(zk_connect) do |zk|
          zk.children(discovery_path).shuffle.each do |broker|
            zktdata = YAML.load(zk.get("#{discovery_path}/#{broker}").first)
            if zktdata["address"] && zktdata["port"]
              brokers_names << "http://#{zktdata['address']}:#{zktdata['port']}/druid/v2/?pretty"
            end
          end
        end
    
        brokers_names
      end

      def send_query_to_druid(query)
        # uri = URI('http://druid-broker.service:8081/druid/v2/')
        uri = druid_url
        http = Net::HTTP.new(uri.host, uri.port)
        request = Net::HTTP::Post.new(uri.path, 'Content-Type' => 'application/json')
        request.body = query.to_json

        response = http.request(request)
        JSON.parse(response.body)
      rescue => e
        { error: e.message }
      end
    end
  end
end